;------------------------------------------------------------------------------
;	Chaotic subsystems (512b)
;	by rudi / dke~rdm
;------------------------------------------------------------------------------
*= $0801
	.word (+), 2015		;line number
	.null $9e, ^start	;sys 4096
+	.word 0			;line end

;------------------------------------------------------------------------------
;	system start at 4096:
;------------------------------------------------------------------------------
start:
   jsr InitBitmap
   jsr BlackBorder

;------------------------------------------------------------------------------
;	plot random configuration
;------------------------------------------------------------------------------
   ldy #0
   ldx #0
L1 adc $d012		;prng
   pha
   and #1
   bne L2
   jsr StoreRegs
   jsr SetPixel
   jsr LoadRegs
L2 pla
   inx
   bne L1

;------------------------------------------------------------------------------
;	generate table for YtabHI
;------------------------------------------------------------------------------
   lda #$20
   sta $e0
   sta $e1
   ldy #0   
   ldx #0
V0 lda $e0
   sta YtabHI,x
   inx
   txa
   and #7
   bne V0
   inc $e0
   inc $e1
   lda $e1
   and #3
   bne V1
   inc $e0
V1 iny
   cpy #25
   bne V0

;------------------------------------------------------------------------------
;	do the cellular automaton
;------------------------------------------------------------------------------
   ldy #0	;Y
fory:
   ldx #0	;X
forx:
   lda #0
   sta Px
   sta Qx
   sta Rx
P  jsr GetPixel
   bcs Q
   lda #1
   sta Px
Q  inx
   jsr GetPixel
   bcs R
   lda #1
   sta Qx
R  inx
   jsr GetPixel
   bcs L4
   lda #1
   sta Rx
L4 dex
   jsr StoreRegs
   iny
   sec
   lda Px
   and Qx
   adc #1
   adc Px
   adc Qx
   adc Rx	;algebraic (rule 169)   
   and #1
   beq L5
   jsr SetPixel
L5 jsr LoadRegs
   cpx #255
   bne forx
   iny
   cpy #199
   bne fory

jmp *

Px .byte $00
Qx .byte $00
Rx .byte $00

;------------------------------------------------------------------------------
;	functions and tables
;------------------------------------------------------------------------------
BlackBorder .proc	;set border color to black
   lda #0
   sta $d020
   rts
.pend

StoreRegs .proc
   stx $fd
   sty $fe
   sta $ff
   rts
.pend

LoadRegs .proc
   ldx $fd
   ldy $fe
   lda $ff
   rts
.pend
	
InitBitmap .proc	;put bitmap at 8192
   lda $d018
   ora #%00001000
   sta $d018

   lda $d011		;enter bitmap mode
   ora #%00100000
   sta $d011

   ldx #0		;clear bitmap
   ldy #$20
L0 stx $2000
   inc L0+1
   bne L0
   inc L0+2
   dey
   bne L0

   lda #1		;set color
   ldx #0
L1 sta $0400,x
   sta $0500,x
   sta $0600,x
   sta $0700,x
   inx
   bne L1
   rts
.pend

PixelAddr = $fb

;------------------------------------------------------------------------------
;SetPixel procedure
;------------------------------------------------------------------------------
SetPixel .proc
   txa
   and #7
   sta $03fc		;temp for x
   txa
   .byte $4a,$4a,$4a, $0a ;3*shr, shl
   tax

   sty $a3
   tya
   and #31		;wrap
   tay
   lda YtabLO,y
   sta PixelAddr	;store low addr
   ldy $a3
   lda YtabHI,y
   sta PixelAddr+1	;store hi addr

   ldy Xtab,x		;load y from xtab,x
   lda (PixelAddr),y
   ldx $03fc		;load temp in x
   ora Mask,x
   sta (PixelAddr),y	;plot

   rts
.pend

;------------------------------------------------------------------------------
;GetPixel procedure (returns bit in accum)
;------------------------------------------------------------------------------
GetPixel .proc
   clc
   .byte $8a,$48,$98,$48 ;txa pha tya pha
   txa
   and #7
   sta $03fc		;temp for x
   txa
   .byte $4a,$4a,$4a, $0a ;3*shr, shl
   tax

   sty $a3
   tya
   and #31		;wrap y (lo)
   tay
   lda YtabLO,y
   sta PixelAddr	;store low addr
   ldy $a3
   lda YtabHI,y
   sta PixelAddr+1	;store hi addr

   ldy Xtab,x		;load y from xtab,x
   lda (PixelAddr),y
   ldx $03fc
   and Mask,x

   jsr ShiftRightReverse

   ror			;to set or not set carry flag
   .byte $68,$a8,$68,$aa	;pla tay pla tax

   rts
.pend

ShiftRightReverse .proc
   cpx #0
   beq SHR7
   cpx #1
   beq SHR6
   cpx #2
   beq SHR5
   cpx #3
   beq SHR4
   cpx #4
   beq SHR3
   cpx #5
   beq SHR2
   cpx #6
   beq SHR1
   cpx #7
   beq SHR0
SHR7 .byte $4a
SHR6 .byte $4a
SHR5 .byte $4a
SHR4 .byte $4a
SHR3 .byte $4a
SHR2 .byte $4a
SHR1 .byte $4a
SHR0 .byte $60	;rts
.pend

;------------------------------------------------------------------------------
;tables
;------------------------------------------------------------------------------
Mask	.byte %10000000
	.byte %01000000
	.byte %00100000
	.byte %00010000
	.byte %00001000
	.byte %00000100
	.byte %00000010
	.byte %00000001

Xtab	.word $0000, $0008, $0010, $0018, $0020, $0028, $0030, $0038
	.word $0040, $0048, $0050, $0058, $0060, $0068, $0070, $0078
	.word $0080, $0088, $0090, $0098, $00A0, $00A8, $00B0, $00B8
	.word $00C0, $00C8, $00D0, $00D8, $00E0, $00E8, $00F0, $00F8
	.word $0100, $0108, $0110, $0118, $0120, $0128, $0130, $0138

YtabLO	.byte $00, $01, $02, $03, $04, $05, $06, $07
	.byte $40, $41, $42, $43, $44, $45, $46, $47
	.byte $80, $81, $82, $83, $84, $85, $86, $87
	.byte $C0, $C1, $C2, $C3, $C4, $C5, $C6, $C7

YtabHI	.byte $20